package de.isse.kiv.refactoring

import org.eclipse.core.resources.IFile
import org.eclipse.core.runtime.IProgressMonitor
import org.eclipse.ltk.core.refactoring.Change
import org.eclipse.ltk.core.refactoring.Refactoring
import org.eclipse.ltk.core.refactoring.RefactoringStatus
import org.eclipse.ui.texteditor.ITextEditor
import org.eclipse.ltk.core.refactoring.CompositeChange
import org.eclipse.ltk.core.refactoring.DocumentChange
import org.eclipse.jface.text.ITextSelection
import de.isse.kiv.resources.ResourceLookup
import scala.collection.mutable.ListBuffer
import de.isse.kiv.resources.ResourceProperties._
import kiv.util.ScalaExtensions._
import org.eclipse.jface.viewers.ISelection
import kiv.printer.prettyprint.xpp
import kiv.util.KivType
import kiv.spec.Theorem
import de.isse.kiv.resources.FileModel
import kiv.parser._
import kiv.expr.Xov
import de.isse.kiv.source.parser.KIVToken
import de.isse.kiv.source.parser.VariableToken
import scala.Left
import scala.Right

// TODO: remove extracttype (0-2 = Function/Predicate/Procedure, 3 = anything)
class CodeExtractRefactoring(var extracttype: Int, editor: ITextEditor, selection: ISelection, file: IFile, fileModel: FileModel) extends Refactoring {

  var editedName: String = _
  var selectedString: String = _
  var selectedLocation: Location = _
  var variableParametersProcedure: PreFpl = _
  var bracketSemicolonLocation: Int = _
  var returntype: String = ""
  var variables: List[Xov] = _
  var hackLocation: Location = _ // is used to find expr or preprog until "(" and "{" and ")" and "}" have locations in expressions and programs
  setSelectionInformation()
  initialise()

  def setSelectionInformation() {
    try {
      selection match {
        case sel: ITextSelection => {
          selectedString = sel.getText
          selectedLocation = new Location(sel.getStartLine, sel.getEndLine, sel.getOffset, sel.getLength)
        }
        case _ =>
      }
    } catch { case exp: Throwable => println(exp) }
  }

  def initialise() {
    val docProv = editor.getDocumentProvider
    if (docProv == null) return
    val document = docProv.getDocument(editor.getEditorInput)

    variables = findPreXovForSelectedLocationFromKivToken(getAllKivTokenInSelectedLocation)

    hackLocation = selectedLocation
    applyHackLocation()

    val option = getPreProgOrPreExprForSelectedLocation
    if (option != null) option match {
      case Right(value: PreExpr) => value.location match {
        case Some(location) => {
          if (hackLocation == selectedLocation) {
            selectedLocation = location
            selectedString = document.get(location.position, location.length)
          }
          val freeVars = value.getFreeVariables(Nil)
          val allVars = value.getAllPreXov
          val varsWithoutFreeVars = allVars.filter { prexov => !freeVars.contains(prexov) }
          variables = varsWithoutFreeVars.map { prexov => prexov.xov }.distinct
          returntype = xpp(value.typ)
          if (extracttype == 3) {
            if (returntype == "bool") extracttype = 1;
            else extracttype = 2;
          }
        }
        case None =>
      }
      case Left(value: PreProg) => value.location match {
        case Some(location) => {
          if (hackLocation == selectedLocation) {
            selectedLocation = location
            selectedString = document.get(location.position, location.length)
          }
          val freeVars = value.getFreeVariables(Nil)
          println("FreeVars: " + freeVars)
          val allVars = value.getAllPreXov
          println("AllVars: " + allVars)
          val varsWithoutFreeVars = allVars.filter { prexov => !freeVars.contains(prexov) }
          variables = varsWithoutFreeVars.map { prexov => prexov.xov }.distinct
          if (extracttype == 3) extracttype = 0
        }
        case None =>
      }

    }
  }

  def applyHackLocation() {
    var i = 0;
    while (selectedString.charAt(i) == '(' || selectedString.charAt(i) == '{') {
      hackLocation = new Location(hackLocation.line, hackLocation.column, hackLocation.position + 1, hackLocation.length - 1)
      i = i + 1
    }
    var j = selectedString.length() - 1
    while (selectedString.charAt(j) == ')' || selectedString.charAt(j) == '}' || (selectedString.charAt(j) == ';' && j == selectedString.length() - 1)) {
      hackLocation = new Location(hackLocation.line, hackLocation.column, hackLocation.position, hackLocation.length - 1)
      j = j - 1
    }
  }

  def checkFinalConditions(x$1: IProgressMonitor): RefactoringStatus = {
    val refactoringStatus = new RefactoringStatus
    refactoringStatus
  }

  def checkInitialConditions(x$1: IProgressMonitor): RefactoringStatus = {
    println("Check initial conditions")
    val refactoringStatus = new RefactoringStatus
    if (selectedString == null || selectedLocation == null) {
      println("Initial Conditions not fulfilled")
      refactoringStatus.addInfo("Nothing selected, which could be extracted")
    }
    //TODO: Add information to the Status
    refactoringStatus
  }

  def createChange(monitor: IProgressMonitor): Change = {
    val docProv = editor.getDocumentProvider
    if (docProv == null) return null
    val document = docProv.getDocument(editor.getEditorInput)
    monitor.beginTask("Creating change...", IProgressMonitor.UNKNOWN);

    var extractListBuffer = new ListBuffer[DocumentChange]();

    val projectModel = ResourceLookup.activeProject.projectModel

    println(variables)

    if (extracttype == 0) {

      val insertionLocation = getInsertionLocationForDeclaration()

      val insertContent = "\n\n" + editedName + ":\n" + editedName + "#" + variableParameterString(variables) + "\n{\n  " + selectedString + "\n};\n"

      val replaceContent = editedName + "#" + variableParameterString(variables) + ";"

      extractListBuffer += new InsertTextChange(document, insertionLocation, editedName, insertContent)
      extractListBuffer += new ReplaceTextChange(document, selectedLocation, editedName, replaceContent)
    }

    def checkString(s: String) = {
      if (s.endsWith(";")) s
      else s + ";"
    }

    def semicolonInSameLine = {
      val lastLineOfSelectedString = getAllKivTokenInSelectedLocation.lastOption match {
        case Some(kivtoken) => kivtoken.location.line
        case None           => -1
      }
      !(fileModel.tokens.filter { token =>
        token.location.line == lastLineOfSelectedString && token.stringPresentation == ";" &&
          token.location.position + token.location.length > selectedLocation.position + selectedLocation.length
      }.isEmpty)
    }

    def insideLargeTheorem = {
      val spec = fileModel.getPreSpecificationOrSequents
      val theorems = spec match {
        case Some(Left(prespec)) => prespec.theorems
        case _                   => Nil
      }
      try {
        val locations = theorems.map { theorem => theorem.preseq.location.get }
        val bigTheorem = locations.filter { loc => loc.position <= selectedLocation.position && (loc.position + loc.length) >= (selectedLocation.position + selectedLocation.length) }
        !bigTheorem.isEmpty
      } catch { case e: Throwable => false }
    }

    println("Semicolon in same Line: " + semicolonInSameLine)
    println("Inside Large Theorem: " + insideLargeTheorem)

    if (extracttype == 1) {

      val declarationLocation = fileModel.findDefinitionToken("predicates") match {
        case Some(kivtoken) => new Location(0, 0, kivtoken.location.position + kivtoken.location.length + 1, 0)
        case None           => new Location(0, 0, 0, 0) //TODO: Use good location
      }

      val declarationContent = "" + editedName + " : " + variableParameterStringForDeclarationOfPredicate(variables) + ";\n"

      val insertionLocation = getInsertionLocationForPredicate

      val insertContent = "" + editedName + ":\n" + editedName + variableParameterStringOp(variables) + " ↔ " + checkString(selectedString) + "\n\n"

      val replaceContent = if (semicolonInSameLine || insideLargeTheorem) editedName + variableParameterStringOp(variables)
      else editedName + variableParameterStringOp(variables) + ";"

      extractListBuffer += new ReplaceTextChange(document, selectedLocation, editedName, replaceContent)
      extractListBuffer += new InsertTextChange(document, insertionLocation, editedName, insertContent)
      extractListBuffer += new InsertTextChange(document, declarationLocation, editedName, declarationContent)
    }

    if (extracttype == 2) {

      val declarationLocation = fileModel.findDefinitionToken("functions") match {
        case Some(kivtoken) => new Location(0, 0, kivtoken.location.position + kivtoken.location.length + 1, 0)
        case None           => new Location(0, 0, 0, 0) //TODO: Use good location
      }

      val declarationContent = "" + editedName + " : " + variableParameterStringForDeclarationOfFunction(variables) + ";\n"

      val insertionLocation = getInsertionLocationForFunction

      val insertContent = "" + editedName + ":\n" + editedName + variableParameterStringOp(variables) + " = " + checkString(selectedString) + "\n\n"

      val replaceContent = if (semicolonInSameLine || insideLargeTheorem) editedName + variableParameterStringOp(variables)
      else editedName + variableParameterStringOp(variables) + ";"

      extractListBuffer += new ReplaceTextChange(document, selectedLocation, editedName, replaceContent)
      extractListBuffer += new InsertTextChange(document, insertionLocation, editedName, insertContent)
      extractListBuffer += new InsertTextChange(document, declarationLocation, editedName, declarationContent)
    }

    new CompositeChange("composite", extractListBuffer.toArray)
  }

  def getName(): String = {
    "CodeExtractRefactoring"
  }

  def variableParameterString(variables: List[Xov]): String = {
    if (variableParametersProcedure == null) ""
    else {
      val listValue = variableParametersProcedure.valueparams.filter { prexov => variables.contains(prexov.xov) }.map { prexov => prexov.xov.xovsym.name }
      val listVar = variableParametersProcedure.varparams.filter { prexov => variables.contains(prexov.xov) }.map { prexov => prexov.xov.xovsym.name }
      val listOut = variableParametersProcedure.outparams.filter { prexov => variables.contains(prexov.xov) }.map { prexov => prexov.xov.xovsym.name }
      "(" + listValue.mkString(", ") + "; " + listVar.mkString(", ") + "; " + listOut.mkString(", ") + ")"
    }
  }

  def variableParameterStringOp(variables: List[Xov]): String = {
    if (variables == null) ""
    else {
      val symbols = variables.map { xov => xov.xovsym.name }
      "(" + symbols.mkString(", ") + ")"
    }
  }

  def variableParameterStringForDeclarationOfFunction(variables: List[Xov]): String = {
    variableParameterStringForDeclarationOfPredicate(variables) + " → " + returntype
  }

  def variableParameterStringForDeclarationOfPredicate(variables: List[Xov]): String = {
    variables.map { xov => xpp(xov.typ) }.mkString(" × ")
  }

  def getAllKivTokenInSelectedLocation: List[KIVToken] = {
    fileModel.tokens.filter { token =>
      token.location.position >= selectedLocation.position &&
        token.location.position + token.location.length <= selectedLocation.position + selectedLocation.length
    }
  }

  def findPreXovForSelectedLocationFromKivToken(tokens: List[KIVToken]): List[Xov] = {
    val prexovList: List[Xov] = tokens.filter { kivToken =>
      kivToken match {
        case variable: VariableToken => true
        case _                       => false
      }
    }.map { kivToken =>
      kivToken match {
        case VariableToken(prexov: PreXov) => prexov.xov
        case _                             => null
      }
    }.distinct
    prexovList
  }

  def getInsertionLocationForPredicate: Location = {
    getInsertionLocationForAxiom
  }

  def getInsertionLocationForFunction: Location = {
    getInsertionLocationForAxiom
  }

  def getInsertionLocationForAxiom: Location = {
    fileModel.findDefinitionToken("axioms") match {
      case Some(kivtoken) => new Location(0, 0, kivtoken.location.position + kivtoken.location.length + 1, 0)
      case None           => new Location(0, 0, 0, 0)
    }
  }

  def getPreProgOrPreExprForSelectedLocation: Either[PreProg, PreExpr] = {
    val spec = fileModel.getPreSpecificationOrSequents
    val list = spec match {
      case Some(Left(prespec)) => prespec.declarations.filter { predeclaration =>
        predeclaration match {
          case operation: PreOperationDeclaration => preprogContainsLocation(operation)
          case _                                  => false
        }
      } ++ prespec.theorems.filter { theorem => theoremContainsLocation(theorem) }
      case Some(Right(theorems)) => theorems.filter { theorem => theoremContainsLocation(theorem) }
      case _                     => Nil
    }
    if (list.size == 1) {
      val preprog = list.head match {
        case operation: PreOperationDeclaration => Some(operation.preprog)
        case theorem: Theorem                   => Some(theorem.preseq)
        case _                                  => None
      }
      val kivtype = preprog match {
        case Some(preprog: PreProg) => findKivTypeForSelectedLocation(preprog)
        case Some(preseq: PreSeq) => {
          val patantList = preseq.patant match {
            case PreFl1(expressions) => expressions
            case _                   => Nil
          }
          val patsucList = preseq.patsuc match {
            case PreFl1(expressions) => expressions
            case _                   => Nil
          }
          findKivTypeForSelectedLocation(patantList ++ patsucList)
        }
        case _ => None
      }
      kivtype match {
        case Some(expr: PreExpr) =>
          println("Found Expression: " + expr + " Location: " + expr.location); return Right(expr)
        case Some(prog: PreProg) =>
          println("Found Prog: " + prog + " Location: " + prog.location); return Left(prog)
        case Some(_)             => println("Anderer Kivtype gefunden als PreExpr oder PreProg")
        case None                => println("Kivtype mit passender Location nicht gefunden!")
      }
    }
    null
  }

  def findKivTypeForSelectedLocation(prog: PreProg): Option[KivType] = { // use selectedLocation after "(" have locations
    val found = prog.location match {
      case Some(location) => location.position <= hackLocation.position &&
        (hackLocation.position + hackLocation.length) <= (location.position + location.length)
      case None => false
    }
    val thisopt: Option[KivType] = if (found) Some(prog) else None
    val subopt = prog match {
      case unaryProg: UnaryProg        => findKivTypeForSelectedLocation(unaryProg.prog)
      case PreAtomic(_, _, expr, prog) => findKivTypeForSelectedLocation(expr).orElse { findKivTypeForSelectedLocation(prog) }
      case binaryProg: BinaryProg      => findKivTypeForSelectedLocation(binaryProg.prog1).orElse { findKivTypeForSelectedLocation(binaryProg.prog2) }
      case interleaving: Interleaving  => findKivTypeForSelectedLocation(interleaving.prog1).orElse { findKivTypeForSelectedLocation(interleaving.prog2) }
      case ifConstruct: IfConstruct => findKivTypeForSelectedLocation(ifConstruct.bxp).orElse {
        findKivTypeForSelectedLocation(ifConstruct.prog1)
      }.orElse {
        ifConstruct.prog2.map { findKivTypeForSelectedLocation }.getOrElse(None)
      }
      case whileConstruct: WhileConstruct => findKivTypeForSelectedLocation(whileConstruct.bxp).orElse { findKivTypeForSelectedLocation(whileConstruct.prog) }
      case PreLoop(_, expr, prog)         => findKivTypeForSelectedLocation(expr).orElse { findKivTypeForSelectedLocation(prog) }
      case PreChoose(_, prevl: PreVl1, expr: PreExpr, prog: PreProg, prog2) => findKivTypeForSelectedLocation(prevl.patvarlist1).orElse { findKivTypeForSelectedLocation(expr) }.orElse {
        findKivTypeForSelectedLocation(prog)
      }.orElse { prog2.map { findKivTypeForSelectedLocation }.getOrElse(None) }
      case PreForall(_, prevl: PreVl1, expr: PreExpr, prog: PreProg) => findKivTypeForSelectedLocation(prevl.patvarlist1).orElse { findKivTypeForSelectedLocation(expr) }.orElse {
        findKivTypeForSelectedLocation(prog)
      }
      case PreLet(_, patvdl: PreVdl1, prog: PreProg) => findKivTypeForSelectedLocation(convertToPreExprList(patvdl.patvdecllist1)).orElse { findKivTypeForSelectedLocation(prog) }
      case PreCall(_, preapl) => findKivTypeForSelectedLocation(preapl.return_parameters).orElse { findKivTypeForSelectedLocation(preapl.value_parameters) }.orElse {
        findKivTypeForSelectedLocation(preapl.variable_parameters)
      }
      case PrePrecall(_, preapl) => findKivTypeForSelectedLocation(preapl.return_parameters).orElse { findKivTypeForSelectedLocation(preapl.value_parameters) }.orElse {
        findKivTypeForSelectedLocation(preapl.variable_parameters)
      }
      case PreBcall(_, preapl, expr) => findKivTypeForSelectedLocation(preapl.return_parameters).orElse { findKivTypeForSelectedLocation(preapl.value_parameters) }.orElse {
        findKivTypeForSelectedLocation(preapl.variable_parameters)
      }.orElse { findKivTypeForSelectedLocation(expr) }
      case PreParasg1(patassignlist1: List[PreAssign]) => findKivTypeForSelectedLocation(convertToPreExprList2(patassignlist1))
      case PreParasg3(patassignlist1: List[PreAssign], _, patassignlist2: List[PreAssign]) => findKivTypeForSelectedLocation(convertToPreExprList2(patassignlist1)).orElse {
        findKivTypeForSelectedLocation(convertToPreExprList2(patassignlist2))
      }
      case PreAssert(_, expr)           => findKivTypeForSelectedLocation(expr)
      case PreAwait(_, expr)            => findKivTypeForSelectedLocation(expr)
      case PreExprprog(_, expr, _)      => findKivTypeForSelectedLocation(expr)
      case PreAnnotation(assertionlist) => findKivTypeForSelectedLocation(getAllPreExprFromAssertionList(assertionlist))
      case _                            => None
    }
    subopt.orElse { thisopt }
  }

  def findKivTypeForSelectedLocation(expr: PreExpr): Option[KivType] = { // use selectedLocation after "(" have locations
    val found = expr.location match {
      case Some(location) => location.position <= hackLocation.position &&
        (hackLocation.position + hackLocation.length) <= (location.position + location.length)
      case None => false
    }
    val thisopt: Option[KivType] = if (found) Some(expr) else None
    val subopt = expr match {
      case unary: UnaryExpr       => findKivTypeForSelectedLocation(unary.fma)
      case primedXov: PrimedXov   => findKivTypeForSelectedLocation(primedXov.fma)
      case binaryExpr: BinaryExpr => findKivTypeForSelectedLocation(binaryExpr.fma1).orElse { findKivTypeForSelectedLocation(binaryExpr.fma2) }
      case progWithPostCondition: ProgWithPostCondition => findKivTypeForSelectedLocation(progWithPostCondition.fma).orElse {
        findKivTypeForSelectedLocation(progWithPostCondition.prog)
      }
      case PreRgbox(_, vl, rely, guar, inv, prog, fma, _) =>
        findKivTypeForSelectedLocation(fma).orElse { findKivTypeForSelectedLocation(guar) }.orElse {
          findKivTypeForSelectedLocation(inv)
        }.orElse { findKivTypeForSelectedLocation(prog) }.orElse {
          findKivTypeForSelectedLocation(rely)
        }.orElse {
          (vl match {
            case prevl1: PreVl1 => findKivTypeForSelectedLocation(prevl1.patvarlist1)
            case _              => None
          })
        }
      case PreRgdia(_, vl, rely, guar, inv, run, prog, fma, _) =>
        findKivTypeForSelectedLocation(fma).orElse { findKivTypeForSelectedLocation(guar) }.orElse {
          findKivTypeForSelectedLocation(inv)
        }.orElse { findKivTypeForSelectedLocation(prog) }.orElse {
          findKivTypeForSelectedLocation(run)
        }.orElse { findKivTypeForSelectedLocation(run) }.orElse {
          findKivTypeForSelectedLocation(rely)
        }.orElse {
          (vl match {
            case prevl1: PreVl1 => findKivTypeForSelectedLocation(prevl1.patvarlist1)
            case _              => None
          })
        }
      case PreAp(expr, listOfExpressions) => findKivTypeForSelectedLocation(expr).orElse { findKivTypeForSelectedLocation(listOfExpressions) }
      case PreLambda(_, vl: PreVl1, expr: PreExpr) => findKivTypeForSelectedLocation(expr).orElse { findKivTypeForSelectedLocation(vl.patvarlist1) }
      case PreAll(_, vl: PreVl1, expr: PreExpr) => findKivTypeForSelectedLocation(expr).orElse { findKivTypeForSelectedLocation(vl.patvarlist1) }
      case PreEx(_, vl: PreVl1, expr: PreExpr) => findKivTypeForSelectedLocation(expr).orElse { findKivTypeForSelectedLocation(vl.patvarlist1) }
      case PreNumexpr(expr: PreExpr) => findKivTypeForSelectedLocation(expr)
      case PreVarprogexpr(_, vl: PreVl1, prog: PreProg, _) => findKivTypeForSelectedLocation(vl.patvarlist1).orElse { findKivTypeForSelectedLocation(prog) }
      case _ => None
    }
    println("aktuelle Expression: " + expr + " Location von aktueller Expression: " + expr.location + " passend: " + found)
    subopt.orElse(thisopt)
  }

  def findKivTypeForSelectedLocation(listExpr: List[PreExpr]): Option[KivType] = {
    val list = listExpr.map { expr => findKivTypeForSelectedLocation(expr) }
    val filteredlist = list.filter { expr =>
      expr match {
        case Some(value) => true
        case None        => false
      }
    }
    if (filteredlist.size == 1) filteredlist.head
    else None
  }

  def convertToPreExprList2(list: List[PreAssign]): List[PreExpr] = {
    list.flatMap {
      case PreAsg(patvar, patterm) =>
        List(patvar, patterm)
      case PreRasg(patvar, _) =>
        List(patvar)
      case PreCasg(patvar, patterm) =>
        List(patvar, patterm)
    }
  }

  def convertToPreExprList(list: List[PreVdecl]): List[PreExpr] = {
    list.flatMap {
      case PreVardecl(patvar, patterm) =>
        List(patvar, patterm)
      case PreRvardecl(patvar) =>
        List(patvar)
    }
  }

  private def getAllPreExprFromAssertionList(assertionlist: List[PreAssertion]): List[PreExpr] = {
    assertionlist.flatMap { assertion => getAllPreExpr(assertion) }
  }

  private def getAllPreExpr(assertion: PreAssertion): List[PreExpr] = {
    assertion match {
      case preContractAssert: PreContractassert => preContractAssert.subst.terms ++ preContractAssert.subst.vars
      case PreWfinvassert(_, invariant: PreExpr, wfbound: PreExpr) => List(invariant, wfbound)
      case PreWfassert(_, wfbound: PreExpr) => List(wfbound)
      case PreStructassert(_, structbound: PreExpr) => List(structbound)
      case PreInvassert(_, invariant: PreExpr) => List(invariant)
      case PreCutassert(_, cutfma: PreExpr) => List(cutfma)
      case _ => Nil
    }
  }

  def getInsertionLocationForDeclaration(): Location = {
    val roundingLocation = getLocationFromRoundingDeclaration
    val insertPosition = roundingLocation match {
      case Some(location) => bracketSemicolonLocation + 2
      case None           => println("no procedure found!!!"); println("Bracket Semiclon: " + bracketSemicolonLocation); selectedLocation.position
    }

    new Location(0, 0, insertPosition, 0) // only the insertPosition is important
  }

  def getLocationFromRoundingDeclaration: Option[Location] = {
    val spec = fileModel.getPreSpecificationOrSequents
    val list = spec match {
      case Some(Left(prespec)) => prespec.declarations.filter { predeclaration =>
        predeclaration match {
          case operation: PreOperationDeclaration => preprogContainsLocation(operation)
          case _                                  => false
        }
      }
      case _ => Nil
    }
    val listOperation = list.filterType[PreOperationDeclaration]
    if (listOperation.size == 1) {
      variableParametersProcedure = listOperation.head.fpl
      listOperation.head.preprog.location
    } else None
  }

  def preprogContainsLocation(operation: PreOperationDeclaration): Boolean = {
    val loc = operation.preprog.location
    println(loc + " --------------------- " + selectedLocation)
    val locOfLastToken = operation.preprog.locations.lastOption
    loc match {
      case Some(loc) => operation.procsym.loc.position <= selectedLocation.position && getPositionOfBracketSemicolon(locOfLastToken.get) >= (selectedLocation.position + selectedLocation.length)
      case None      => false
    }
  }

  def theoremContainsLocation(theorem: Theorem): Boolean = {
    val loc = if (theorem.preseq != null) theorem.preseq.location else None
    println("Theorem Preseq Location: " + loc + " --------------------- " + selectedLocation)
    loc match {
      case Some(loc) => loc.position <= selectedLocation.position && (loc.position + loc.length + 5) >= (selectedLocation.position + selectedLocation.length)
      case None      => false
    }
  }

  def getPositionOfBracketSemicolon(lastTokenLocation: Location): Int = { // searches for procedure end token "}" followed by token ";" and returns the location.position of "}" token
    val tokens = fileModel.tokens
    val lastToken = tokens.find { token => token.location == lastTokenLocation }
    val index = lastToken match {
      case Some(token) => tokens.indexOf(token)
      case None        => 0
    }
    val splittedTokens = tokens.takeRight(tokens.size - index)
    val bracketToken = splittedTokens.find { token => token.stringPresentation == "}" && nextToken(splittedTokens, token).stringPresentation == ";" }
    if (bracketToken.size == 1) { bracketSemicolonLocation = bracketToken.head.location.position; bracketSemicolonLocation }
    else lastTokenLocation.position
  }

  private def nextToken(list: List[KIVToken], token: KIVToken): KIVToken = {
    val index = list.indexOf(token)
    list(index + 1)
  }
}
