package de.isse.kiv.refactoring

import org.eclipse.core.resources.IFile
import org.eclipse.core.runtime.IProgressMonitor
import org.eclipse.ltk.core.refactoring.Change
import org.eclipse.ltk.core.refactoring.Refactoring
import org.eclipse.ltk.core.refactoring.RefactoringStatus
import org.eclipse.ui.texteditor.ITextEditor
import org.eclipse.ltk.core.refactoring.CompositeChange
import org.eclipse.jface.text.IDocument
import org.eclipse.ui.texteditor.IDocumentProvider
import org.eclipse.ui.editors.text.TextFileDocumentProvider
import org.eclipse.ltk.core.refactoring.DocumentChange
import de.isse.kiv.resources.ResourceLookup
import scala.collection.mutable.ListBuffer
import de.isse.kiv.resources.ResourceProperties._
import org.eclipse.jface.viewers.ISelection
import org.eclipse.jface.text.ITextSelection
import de.isse.kiv.resources.FileModel
import kiv.parser._

class CodeMoveRefactoring(editor: ITextEditor, selection: ISelection, file: IFile) extends Refactoring {

  var selectedString: String = _
  var selectedLocation: Location = _
  val relatedSpecFiles: List[IFile] = getAllRelatedSpecs
  var selectedSpecFile: IFile = _
  var locationOfInsertReplace: Location = _
  setSelectionInformation()

  def setSelectionInformation() {
    try {
      selection match {
        case sel: ITextSelection => {
          selectedString = sel.getText
          selectedLocation = new Location(sel.getStartLine, sel.getEndLine, sel.getOffset, sel.getLength)
        }
        case _ =>
      }
    } catch { case exp: Throwable => println(exp) }
  }

  def checkFinalConditions(x$1: IProgressMonitor): RefactoringStatus = {
    val refactoringStatus = new RefactoringStatus
    refactoringStatus
  }

  def checkInitialConditions(x$1: IProgressMonitor): RefactoringStatus = {
    println("Check initial conditions")
    val refactoringStatus = new RefactoringStatus
    if (selectedString == null || selectedLocation == null) {
      println("Initial Conditions not fulfilled")
      refactoringStatus.addInfo("Nothing selected, which could be extracted")
    }
    //TODO: Add information to the Status
    refactoringStatus
  }

  def getName(): String = {
    "CodeMoveRefactoring"
  }

  def createChange(monitor: IProgressMonitor): Change = {
    val docProv = editor.getDocumentProvider
    if (docProv == null) return null
    val document = docProv.getDocument(editor.getEditorInput)
    monitor.beginTask("Creating change...", IProgressMonitor.UNKNOWN);

    var moveListBuffer = new ListBuffer[DocumentChange]();

    val declarationReplaceContent = getDeclarationContent;
    val replaceContent = selectedString
    val destinationDocument = getDocumentFromDestinationFile()
    val locationOfDeclarationContent: Location = null
    val locationOfReplaceContent: Location = selectedLocation
    val locationOfInsertDeclaration: Location = findInsertDeclarationLocation
    locationOfInsertReplace = findInsertReplaceLocation

    //moveListBuffer += new ReplaceTextChange(document,locationOfDeclarationContent,"","")
    moveListBuffer += new ReplaceTextChange(document, locationOfReplaceContent, "", "")
    //moveListBuffer += new InsertTextChange(destinationDocument,locationOfInsertDeclaration,"","\n"+declarationReplaceContent)
    moveListBuffer += new InsertTextChange(destinationDocument, locationOfInsertReplace, "", "\n\nn" + replaceContent + "\n")

    val change = new CompositeChange("composite", moveListBuffer.toArray)
    change

  }

  def findInsertReplaceLocation: Location = {
    val fileModel = selectedSpecFile.fileModel
    fileModel.tokenizeTokensForFile(selectedSpecFile)
    val declarationsToken = fileModel.findDefinitionToken("declarations")
    declarationsToken match {
      case Some(token) => return new Location(0, 0, token.location.position + token.location.length, 0)
      case None        => return new Location(0, 0, 0, 0)
    }
  }

  def findInsertDeclarationLocation: Location = {
    val fileModel = selectedSpecFile.fileModel
    fileModel.tokenizeTokensForFile(selectedSpecFile)
    val declarationsToken = fileModel.findDefinitionToken("procedures")
    declarationsToken match {
      case Some(token) => return new Location(0, 0, token.location.position + token.location.length, 0)
      case None        => return new Location(0, 0, 0, 0)
    }
  }

  def getDeclarationContent: String = {
    ""
  }

  def getRangeFromInsertLocation: Option[Range] = {
    if (locationOfInsertReplace.position != 0)
      Some(new Range(locationOfInsertReplace.position, locationOfInsertReplace.position + locationOfInsertReplace.length, 1))
    else None
  }

  def getAllRelatedSpecs = {
    val projectModel = ResourceLookup.activeProject.projectModel
    (getAllFilesFromSubSpecs(projectModel.subSpecs, file.specName) ++ getAllFilesFromSubSpecs(projectModel.subSpecsReversed, file.specName)).distinct
  }

  def getAllFilesFromSubSpecs(map: Map[String, List[String]], name: String): List[IFile] = {
    val subSpecs = map(name)
    if (subSpecs.isEmpty) getFilesFromSpecNames(subSpecs)
    else getFilesFromSpecNames(subSpecs) ++ subSpecs.flatMap { subSpec => getAllFilesFromSubSpecs(map, subSpec) }
  }

  def getFilesFromSpecNames(names: List[String]): List[IFile] = {
    val filesOrLibraries = names.map { name => ResourceLookup.specFileOrLibrary(name) }
    var listbuffer = new ListBuffer[IFile]()
    filesOrLibraries.foreach(fileOrLibrary => fileOrLibrary match {
      case Left(file)       => listbuffer += file
      case Right(filestore) =>
    })
    listbuffer.toList
  }

  def getDocumentFromDestinationFile(): IDocument = {
    if (selectedSpecFile == null) return null;
    val provider: IDocumentProvider = new TextFileDocumentProvider();
    provider.connect(selectedSpecFile);
    val document = provider.getDocument(selectedSpecFile)
    document
  }
}
