package de.isse.kiv.resources

import org.eclipse.core.resources.IResource
import org.eclipse.core.resources.IContainer
import org.eclipse.core.resources.IFile
import org.eclipse.core.resources.IResourceDelta

object FileList {
  val NO_FILES: Set[IFile] = Set()

  def collect(resource: IResource): Set[IFile] = {
    if(resource == null || !resource.exists)
      return NO_FILES

    resource match {
      case dir: IContainer =>
        dir.members.toSet.flatMap(collect)
      case file: IFile =>
        Set(file)
    }
  }

  def collectDelta(delta: IResourceDelta): Set[IFile] = {
    if(delta == null)
      return NO_FILES

    val resource = delta.getResource

    if(resource == null || !resource.exists)
      return NO_FILES

    resource match {
      case null =>
        Set()
      case dir: IContainer =>
        delta.getAffectedChildren.toSet.flatMap(collectDelta)
      case file: IFile =>
        Set(file)
    }
  }
}
