package de.isse.kiv.resources

import org.eclipse.core.resources.IFile
import org.eclipse.core.resources.IProject
import org.eclipse.core.resources.IResource
import org.eclipse.core.resources.refresh.IRefreshMonitor
import org.eclipse.core.runtime.CoreException
import org.eclipse.core.runtime.IPath
import org.eclipse.core.runtime.QualifiedName
import org.eclipse.ui.texteditor.ITextEditor
import org.eclipse.ui.IFileEditorInput
import de.isse.kiv.ui.editors.FileEditor

object ResourceProperties {

  val QUALIFIER = "de.isse.kiv"
  val MODEL_PROPERTY = new QualifiedName(QUALIFIER, "model")
  val MONITOR_PROPERTY = new QualifiedName(QUALIFIER, "monitor")

  val DEVGRAPH = "devgraph"
  val PROJECT_INFO = "projectinfo.ppl"

  val SPECS = "specs"
  val DOC = "doc"
  val CONFIG = "config"

  implicit def toResourceProperties(resource: IResource) = new ResourceProperties(resource)
}

class ResourceProperties(resource: IResource) {
  import ResourceProperties._

  private def model(init: => AnyRef): AnyRef = {
    if (resource == null)
      return null
    try {
      var obj = resource.getSessionProperty(MODEL_PROPERTY)
      if (obj == null) {
        obj = init
        resource.setSessionProperty(MODEL_PROPERTY, obj)
      }
      obj
    } catch {
      case e: CoreException => null
    }
  }

  def fileModel: FileModel = {
    model(new FileModel).asInstanceOf[FileModel]
  }

  def projectModel: ProjectModel = {
    model(new ProjectModel).asInstanceOf[ProjectModel]
  }

  def hasMonitor: Boolean = {
    try {
      resource.getSessionProperty(MONITOR_PROPERTY) != null
    } catch {
      case e: CoreException => false
    }
  }

  def setMonitor(monitor: IRefreshMonitor) {
    try {
      resource.setSessionProperty(MONITOR_PROPERTY, monitor)
    } catch {
      case e: CoreException =>
    }
  }

  def unsetMonitor: IRefreshMonitor = {
    try {
      val monitor = resource.getSessionProperty(MONITOR_PROPERTY).asInstanceOf[IRefreshMonitor]
      resource.setSessionProperty(MONITOR_PROPERTY, null)
      monitor
    } catch {
      case e: CoreException => null
    }
  }

  def devgraphFile: IFile = resource.asInstanceOf[IProject].getFile(DEVGRAPH)

  def projectLocation: IPath = {
    devgraphFile.getLocation.removeLastSegments(1)
  }

  def specName: String = {
    resource.getParent.getName
  }

  def isSpecification: Boolean = {
    resource.getName == "specification.utf8"
  }

  def isSequents : Boolean = {
    resource.getName == "sequents.utf8"
  }
}
