package de.isse.kiv.source

import org.eclipse.jface.text.DefaultTextHover
import org.eclipse.jface.text.IDocument
import org.eclipse.jface.text.ITextHover
import org.eclipse.jface.text.contentassist.ContentAssistant
import org.eclipse.jface.text.contentassist.IContentAssistant
import org.eclipse.jface.text.hyperlink.IHyperlinkDetector
import org.eclipse.jface.text.hyperlink.IHyperlinkPresenter
import org.eclipse.jface.text.hyperlink.MultipleHyperlinkPresenter
import org.eclipse.jface.text.presentation.IPresentationReconciler
import org.eclipse.jface.text.presentation.PresentationReconciler
import org.eclipse.jface.text.reconciler.IReconciler
import org.eclipse.jface.text.reconciler.MonoReconciler
import org.eclipse.jface.text.rules.DefaultDamagerRepairer
import org.eclipse.jface.text.source.IAnnotationHover
import org.eclipse.jface.text.source.ISourceViewer
import org.eclipse.jface.text.source.SourceViewerConfiguration
import org.eclipse.swt.graphics.RGB
import org.eclipse.ui.texteditor.ITextEditor
import de.isse.kiv.ui.editors.FileEditor
import de.isse.kiv.refactoring.KIVRefactoring
import de.isse.kiv.source.parser.KIVTokenizer

class Configuration(editor: FileEditor) extends SourceViewerConfiguration {
  override def getAnnotationHover(sourceViewer: ISourceViewer): IAnnotationHover = {
    new ErrorHover(editor)
  }

  override def getReconciler(sourceViewer: ISourceViewer): IReconciler = {
    new MonoReconciler(new ReconcilingStrategy(editor), false)
  }

  override def getContentAssistant(sourceViewer: ISourceViewer): IContentAssistant = {
    val ca = new ContentAssistant()
    ca.setContentAssistProcessor(new SymbolProcessor(editor), IDocument.DEFAULT_CONTENT_TYPE)
    ca.setAutoActivationDelay(1)
    ca.enableAutoActivation(true)
    ca
  }

  override def getHyperlinkDetectors(sourceViewer: ISourceViewer): Array[IHyperlinkDetector] = {
    val refactoring = new KIVRefactoring(editor, sourceViewer)
    Array(new HyperlinkDetector(editor, refactoring))
  }

  override def getHyperlinkPresenter(sourceViewer: ISourceViewer): IHyperlinkPresenter = {
    return new MultipleHyperlinkPresenter(new RGB(255, 0, 0))
  }

  override def getPresentationReconciler(sourceViewer: ISourceViewer): IPresentationReconciler = {
    val pr = new PresentationReconciler()
    val ddr = new DefaultDamagerRepairer(new KIVTokenizer())
    
    pr.setRepairer(ddr, IDocument.DEFAULT_CONTENT_TYPE)
    pr.setDamager(ddr, IDocument.DEFAULT_CONTENT_TYPE)
    pr
  }

  override def getTextHover(sourceViewer: ISourceViewer, s: String): ITextHover = {
    new TypeHover(editor)
  }
}
