package de.isse.kiv.source

import org.eclipse.core.resources.IFile
import org.eclipse.jface.text.IDocument
import org.eclipse.jface.text.IRegion
import org.eclipse.jface.text.reconciler.DirtyRegion
import org.eclipse.jface.text.reconciler.IReconcilingStrategy
import org.eclipse.ui.IEditorInput
import org.eclipse.ui.part.FileEditorInput
import org.eclipse.ui.texteditor.ITextEditor
import scala.collection.JavaConversions._
import de.isse.kiv.ui.Console
import de.isse.kiv.resources.ResourceLookup
import java.net.URI
import org.eclipse.ui.ide.FileStoreEditorInput
import org.eclipse.core.filesystem.IFileStore
import org.eclipse.core.filesystem.EFS
import de.isse.kiv.resources.ResourceProperties.toResourceProperties
import kiv.util.MultiGraph
import kiv.communication.Node
import kiv.util.DefaultEdge
import kiv.printer.prettyprint.xpp
import kiv.spec.Spec
import kiv.parser.PreSpecification
import kiv.spec.Theorem
import de.isse.kiv.source.parser.KIVTokenizer
import de.isse.kiv.source.parser.KIVParser

class ReconcilingStrategy(editor: ITextEditor) extends IReconcilingStrategy {

  private var document: IDocument = _

  override def setDocument(document: IDocument) {
    if (this.document == document) return
    this.document = document
    reconcile(null)
  }

  override def reconcile(dirtyRegion: DirtyRegion, subRegion: IRegion) {
    reconcile(dirtyRegion)
  }

  override def reconcile(partition: IRegion) {
    val file = ResourceLookup.editedFile(editor)

    if (file != null) {
      val parsed = KIVParser.parse(document.get, file)
      parsed match {
        case prespec: PreSpecification =>
          KIVTokenizer.parse(document.get, file, prespec)
        case theoremlist: List[Theorem @unchecked] if theoremlist.forall(_.isInstanceOf[Theorem]) =>
          KIVTokenizer.parse(document.get, file, theoremlist)
        case _ => KIVTokenizer.parse(document.get, file)
      }
    } else
      Console.error.println("You have to import this file to get error markers and file outline")
  }
}
