package de.isse.kiv.source

import org.eclipse.jface.text.rules.ITokenScanner
import org.eclipse.jface.text.rules.IToken
import org.eclipse.jface.text.IDocument
import org.eclipse.jface.text.rules.Token
import org.eclipse.jface.text.TextAttribute
import de.isse.kiv.ui.Console
import java.io.StringReader
import kiv.parser.Terminals._
import kiv.parser.Terminal
import java.util.ArrayList
import org.eclipse.jface.text.rules.ICharacterScanner
import kiv.parser.Terminals
import org.eclipse.swt.SWT
import de.isse.kiv.resources.ResourceLookup
import org.eclipse.ui.editors.text.TextEditor
import org.eclipse.ui.PlatformUI
import org.eclipse.core.resources.IFile
import de.isse.kiv.resources.ResourceProperties._
import de.isse.kiv.resources.FileModel
import de.isse.kiv.resources.ResourceLookup
import kiv.spec.Theorem
import kiv.spec.Spec
import scala.collection.mutable.ListBuffer
import de.isse.kiv.KIVPlugin
import de.isse.kiv.Settings
import org.eclipse.swt.graphics.Color
import org.eclipse.jface.util.IPropertyChangeListener
import org.eclipse.jface.util.PropertyChangeEvent

object TokenMap {

  val KEYWORD = new Token(readTextAttributeFromPrefernceStore(Settings.KEYWORD))
  val PROGRAM = new Token(readTextAttributeFromPrefernceStore(Settings.PROGRAM_CONSTRUCT))
  val PROC = new Token(readTextAttributeFromPrefernceStore(Settings.PROCEDURE))
  val SORT = new Token(readTextAttributeFromPrefernceStore(Settings.SORT))
  val VARIABLE = new Token(readTextAttributeFromPrefernceStore(Settings.VARIABLE))
  val OP = new Token(readTextAttributeFromPrefernceStore(Settings.OP))
  val BOLD_OP = new Token(readTextAttributeFromPrefernceStore(Settings.OPERATOR))
  val COMMENT = new Token(readTextAttributeFromPrefernceStore(Settings.COMMENT))
  val STRING = new Token(readTextAttributeFromPrefernceStore(Settings.STRING))
  val NUMBER = new Token(readTextAttributeFromPrefernceStore(Settings.NUMBER))
  val THEOREM = new Token(readTextAttributeFromPrefernceStore(Settings.THEOREM))
  val SPECIFICATION = new Token(readTextAttributeFromPrefernceStore(Settings.SPECIFICATION))
  val ERROR = {
    val color = new Color(null, 0xFF, 0, 0)
    val attr = new TextAttribute(color, null, 0)
    new Token(attr)
  }

  val DEFAULT = new Token(null)

  def get(n: Int) = {
    val res = tokens(n)
    if (res == null) DEFAULT else res
  }

  val tokens = new Array[IToken](NAMES.length)
  tokens(Terminals.EOF) = Token.EOF

  tokens(T_INSTANTIATE) = KEYWORD
  tokens(T_END_INSTANTIATE) = KEYWORD
  tokens(T_GENERIC_DATA_SPECIFICATION) = KEYWORD
  tokens(T_RENAME) = KEYWORD
  tokens(T_END_GENERIC_DATA_SPECIFICATION) = KEYWORD
  tokens(T_DATA_SPECIFICATION) = KEYWORD
  tokens(T_GENERIC_DATA_SPECIFICATION) = KEYWORD
  tokens(T_END_GENERIC) = KEYWORD
  tokens(T_PARAMETER) = KEYWORD
  tokens(T_END_GENERIC_DATA_SPECIFICATION) = KEYWORD
  tokens(T_SIZE_FUNCTIONS) = KEYWORD
  tokens(T_GENERATED_BY) = KEYWORD
  tokens(T_END_DATA_SPECIFICATION) = KEYWORD
  tokens(T_GENERIC_SPECIFICATION) = KEYWORD
  tokens(T_PARAMETER) = KEYWORD
  tokens(T_TARGET) = KEYWORD
  tokens(T_ORDER_PREDICATES) = KEYWORD
  tokens(T_DATA_TYPES) = KEYWORD
  tokens(T_BY_MORPHISM) = KEYWORD
  tokens(T_BY_MAPPING) = KEYWORD
  tokens(T_ASM_SPECIFICATION) = KEYWORD
  tokens(T_DATA_ASM_SPECIFICATION) = KEYWORD
  tokens(T_AXIOMS) = KEYWORD
  tokens(T_CONSTANTS) = KEYWORD
  tokens(T_END_ASM_SPECIFICATION) = KEYWORD
  tokens(T_END_DATA_ASM_SPECIFICATION) = KEYWORD
  tokens(T_END_ENRICH) = KEYWORD
  tokens(T_END_ACTUALIZE) = KEYWORD
  tokens(T_END_RENAME) = KEYWORD
  tokens(T_END_SPECIFICATION) = KEYWORD
  tokens(T_ACTUALIZE) = KEYWORD
  tokens(T_ENRICH) = KEYWORD
  tokens(T_FINAL_STATE) = KEYWORD
  tokens(T_INITIAL_STATE) = KEYWORD
  tokens(T_PATTERNS) = KEYWORD
  tokens(T_PATTERN) = KEYWORD
  tokens(T_RULE) = KEYWORD
  tokens(T_APPLYRULE_1) = KEYWORD
  tokens(T_APPLYRULE_11) = KEYWORD
  tokens(T_APPLYRULE_ALWAYS) = KEYWORD
  tokens(T_APPLYRULE_ONCE) = KEYWORD
  tokens(T_APPLYRULE_UNIQUE) = KEYWORD
  tokens(T_LEMMAS) = KEYWORD
  tokens(T_THEOREMS) = KEYWORD
  tokens(T_MODULE) = KEYWORD
  tokens(T_ASM_REFINEMENT) = KEYWORD
  tokens(T_ASM_REDUCTION) = KEYWORD
  tokens(T_END_ASM_REDUCTION) = KEYWORD
  tokens(T_REDUCED_DATA_ASM) = KEYWORD
  tokens(T_END_REDUCED_DATA_ASM) = KEYWORD
  tokens(T_END_MODULE) = KEYWORD
  tokens(T_END_ASM_REFINEMENT) = KEYWORD
  tokens(T_EXPORT) = KEYWORD
  tokens(T_IMPORT) = KEYWORD
  tokens(T_SORTS) = KEYWORD
  tokens(T_FUNCTIONS) = KEYWORD
  tokens(T_PREDICATES) = KEYWORD
  tokens(T_PARTIAL_FUNCTIONS) = KEYWORD
  tokens(T_PARTIAL_PREDICATES) = KEYWORD
  tokens(T_CONSTANTS) = KEYWORD
  tokens(T_INDUCTION) = KEYWORD
  tokens(T_AXIOMS) = KEYWORD
  tokens(T_SPECIFICATION) = KEYWORD
  tokens(T_DATAASM_OPTIONS) = KEYWORD
  tokens(T_DATAASM_OPTION_COMPLETE_STATE) = KEYWORD
  tokens(T_STATE_VARIABLES) = KEYWORD
  tokens(T_GHOST_STATE_VARIABLES) = KEYWORD
  tokens(T_USEDFOR) = KEYWORD
  tokens(T_USING) = KEYWORD
  tokens(T_VARIABLES) = KEYWORD
  tokens(T_ASM_RULE) = KEYWORD
  tokens(T_DECLARATIONS) = KEYWORD
  tokens(T_PROCEDURES) = KEYWORD
  tokens(T_NONFUNCTIONAL) = KEYWORD
  tokens(T_NONDETERMINISTIC) = KEYWORD
  tokens(T_FLEXIBLE) = KEYWORD
  tokens(T_ABSTRACTION_RELATION) = KEYWORD
  tokens(T_INTERNAL_EQUIVALENCE) = KEYWORD
  tokens(T_INVARIANTS) = KEYWORD
  tokens(T_IDLE_PREDICATE) = KEYWORD
  tokens(T_RUNS_PREDICATE) = KEYWORD
  tokens(T_INVARIANT_EXPRESSIONS) = KEYWORD
  tokens(T_SEQUENTIAL) = KEYWORD
  tokens(T_CONCURRENT) = KEYWORD
  tokens(T_THREAD_ID) = KEYWORD
  tokens(T_GUARANTEE) = KEYWORD
  tokens(T_OWNERSHIP_FIELDS) = KEYWORD
  tokens(T_OWNERSHIP_HIERARCHY) = KEYWORD
  // tokens(T_ASM_OWNERSHIP_HIERARCHY) = KEYWORD
  tokens(T_OWNED_BY) = KEYWORD
  tokens(T_INITIALIZATION_PROCEDURE) = KEYWORD
  tokens(T_INTERNAL_PROCEDURES) = KEYWORD
  tokens(T_CRASH_PREDICATE) = KEYWORD
  tokens(T_LOCAL_CRASH_EFFECT) = KEYWORD
  tokens(T_INTRODUCING_SYNC_CLASSIFIER) = KEYWORD
  tokens(T_SYNCED_WHEN) = KEYWORD
  tokens(T_SYNC_IDENTIFIER) = KEYWORD
  tokens(T_LOCAL_CRASH_EFFECT) = KEYWORD
  tokens(T_GLOBAL_CRASH_EFFECT) = KEYWORD
  tokens(T_WITH_CRASH_NEUTRALITY) = KEYWORD
  tokens(T_INTRODUCING_SYNC_PREDICATE) = KEYWORD
  tokens(T_SYNC_PREDICATE) = KEYWORD
  tokens(T_INITIALLY) = KEYWORD
  tokens(T_RESTRICTION) = KEYWORD
  tokens(T_RECOVERY_PROCEDURE) = KEYWORD
  tokens(T_DECLARATIONS) = KEYWORD
  tokens(T_ACTF) = KEYWORD

  tokens(T_IF) = PROGRAM
  tokens(T_ITLIF) = PROGRAM
  tokens(T_THEN) = PROGRAM
  tokens(T_ELSE) = PROGRAM
  tokens(T_LET) = PROGRAM
  tokens(T_ABORT) = PROGRAM
  tokens(T_PBLOCKED) = PROGRAM
  tokens(T_SKIP) = PROGRAM
  tokens(T_IN) = PROGRAM
  tokens(T_CHOOSE) = PROGRAM
  tokens(T_IFNONE) = PROGRAM
  tokens(T_WHILE) = PROGRAM
  tokens(T_ITLWHILE) = PROGRAM
  tokens(T_DO) = PROGRAM
  tokens(T_WITH) = PROGRAM
  tokens(T_LOOP) = PROGRAM
  tokens(T_PRECONDITION) = KEYWORD
  tokens(T_POSTCONDITION) = KEYWORD
  tokens(T_SYNCHRONIZING) = KEYWORD
  tokens(T_ASSIGN_RANDOM) = PROGRAM
  tokens(T_AWAIT) = PROGRAM
  tokens(T_GLEICH_RANDOM) = PROGRAM
  tokens(T_ATOMIC) = PROGRAM
  tokens(T_LEFT_MOVER) = PROGRAM
  tokens(T_RIGHT_MOVER) = PROGRAM
  tokens(T_BOTH_MOVER) = PROGRAM
  tokens(T_TRY) = PROGRAM
  tokens(T_CATCH) = PROGRAM
  tokens(T_DEFAULT) = PROGRAM
  tokens(T_THROW) = PROGRAM
  tokens(T_OR) = PROGRAM

  // Annotations
  tokens(T_INVARIANT) = PROGRAM
  tokens(T_WFBOUND) = PROGRAM
  tokens(T_ASSERT) = PROGRAM
  tokens(T_USE_CONTRACT) = PROGRAM
  tokens(T_CALL) = PROGRAM

  tokens(T_PROC) = PROC

  tokens(T_COMMENT) = COMMENT

  tokens(T_XOV) = VARIABLE
  tokens(T_PRIMEDXOV) = VARIABLE
  tokens(T_PRIME) = VARIABLE
  tokens(T_DPRIME) = VARIABLE
  tokens(T_OLDXOV) = BOLD_OP
  tokens(T_SORT_OR_XOV) = VARIABLE

  tokens(T_SORT) = SORT

  tokens(T_FCT) = OP
  tokens(T_INFIXFCTL10) = OP
  tokens(T_INFIXFCTL11) = OP
  tokens(T_INFIXFCTL12) = OP
  tokens(T_INFIXFCTL13) = OP
  tokens(T_INFIXFCTL14) = OP
  tokens(T_INFIXFCTL15) = OP
  tokens(T_INFIXFCTL1) = OP
  tokens(T_INFIXFCTL2) = OP
  tokens(T_INFIXFCTL3) = OP
  tokens(T_INFIXFCTL4) = OP
  tokens(T_INFIXFCTL5) = OP
  tokens(T_INFIXFCTL6) = OP
  tokens(T_INFIXFCTL7) = OP
  tokens(T_INFIXFCTL8) = OP
  tokens(T_INFIXFCTL9) = OP
  tokens(T_INFIXFCTR10) = OP
  tokens(T_INFIXFCTR11) = OP
  tokens(T_INFIXFCTR12) = OP
  tokens(T_INFIXFCTR13) = OP
  tokens(T_INFIXFCTR14) = OP
  tokens(T_INFIXFCTR15) = OP
  tokens(T_INFIXFCTR1) = OP
  tokens(T_INFIXFCTR2) = OP
  tokens(T_INFIXFCTR3) = OP
  tokens(T_INFIXFCTR4) = OP
  tokens(T_INFIXFCTR5) = OP
  tokens(T_INFIXFCTR6) = OP
  tokens(T_INFIXFCTR7) = OP
  tokens(T_INFIXFCTR8) = OP
  tokens(T_INFIXFCTR9) = OP
  tokens(T_POSTFIXFCT) = OP
  tokens(T_PRAEFIXFCT) = OP
  tokens(T_PRAEFIXPRD) = OP

  tokens(T_LAMBDA) = BOLD_OP
  tokens(T_EX) = BOLD_OP
  tokens(T_ALL) = BOLD_OP
  tokens(T_SEQUENT) = BOLD_OP
  tokens(T_LASTSTEP) = BOLD_OP
  tokens(T_BLOCKED) = BOLD_OP
  tokens(T_NUMCHAR) = STRING
  tokens(T_NUMSTRING) = STRING
  tokens(T_NUMINT) = NUMBER
  tokens(T_NUMNAT) = NUMBER
  tokens(T_UNTIL) = BOLD_OP
  tokens(T_UNLESS) = BOLD_OP

  private def readTextAttributeFromPrefernceStore(tokenid: String): TextAttribute = {

    val plugin = KIVPlugin.getDefault
    val preferences = plugin.getPreferenceStore

    val bold = preferences.getBoolean(Settings.TOKEN_NAME_MAP.get(tokenid).get + Settings.SYNTAX_BOLD)
    val italic = preferences.getBoolean(Settings.TOKEN_NAME_MAP.get(tokenid).get + Settings.SYNTAX_ITALIC)
    val strikethrough = preferences.getBoolean(Settings.TOKEN_NAME_MAP.get(tokenid).get + Settings.SYNTAX_STRIKETHROUGH)
    val underlined = preferences.getBoolean(Settings.TOKEN_NAME_MAP.get(tokenid).get + Settings.SYNTAX_UNDERLINED)

    val swtStyle: Int = (if (bold) SWT.BOLD else 0) | (if (italic) SWT.ITALIC else 0) | (if (strikethrough) TextAttribute.STRIKETHROUGH else 0) | (if (underlined) TextAttribute.UNDERLINE else 0)

    val colorString = preferences.getString(Settings.TOKEN_NAME_MAP.get(tokenid).get + Settings.SYNTAX_COLOR)
    val rgb = colorString.split(",")
    val r = rgb(0).trim.toInt
    val g = rgb(1).trim.toInt
    val b = rgb(2).trim.toInt
    val color = new Color(null, r, g, b)

    new TextAttribute(color, null, swtStyle)
  }

 def readSyntaxhighlightFromPreferenceStore {
    KEYWORD.setData(readTextAttributeFromPrefernceStore(Settings.KEYWORD))
    PROGRAM.setData((readTextAttributeFromPrefernceStore(Settings.PROGRAM_CONSTRUCT)))
    PROC.setData((readTextAttributeFromPrefernceStore(Settings.PROCEDURE)))
    SORT.setData((readTextAttributeFromPrefernceStore(Settings.SORT)))
    VARIABLE.setData((readTextAttributeFromPrefernceStore(Settings.VARIABLE)))
    OP.setData((readTextAttributeFromPrefernceStore(Settings.OP)))
    BOLD_OP.setData((readTextAttributeFromPrefernceStore(Settings.OPERATOR)))
    COMMENT.setData(readTextAttributeFromPrefernceStore(Settings.COMMENT))

  }
}
