package de.isse.kiv.ui

import org.eclipse.core.runtime.IPath
import org.eclipse.jface.preference.BooleanFieldEditor
import org.eclipse.jface.preference.FieldEditor
import org.eclipse.jface.preference.FieldEditorPreferencePage
import org.eclipse.jface.preference.StringFieldEditor
import org.eclipse.swt.SWT
import org.eclipse.swt.events.FocusAdapter
import org.eclipse.swt.events.FocusEvent
import org.eclipse.swt.events.SelectionAdapter
import org.eclipse.swt.events.SelectionEvent
import org.eclipse.swt.layout.GridData
import org.eclipse.swt.layout.GridLayout
import org.eclipse.swt.widgets.Button
import org.eclipse.swt.widgets.Composite
import org.eclipse.swt.widgets.Control
import org.eclipse.swt.widgets.Group
import org.eclipse.swt.widgets.Label
import org.eclipse.swt.widgets.Text
import org.eclipse.ui.IWorkbench
import org.eclipse.ui.IWorkbenchPreferencePage
import de.isse.kiv.KIVPlugin
import de.isse.kiv.KIVSystem
import de.isse.kiv.Settings
import de.isse.kiv.resources.ResourceLookup
import PreferencePage._
import org.eclipse.swt.widgets.Listener
import org.eclipse.swt.widgets.Event
import org.eclipse.swt.events.SelectionListener
import org.eclipse.core.runtime.Platform
import org.eclipse.swt.widgets.Display
import org.eclipse.swt.widgets.Shell
import org.eclipse.swt.widgets.FileDialog
import org.eclipse.ui.PlatformUI
import java.io.InputStream
import java.io.File
import java.io.FileOutputStream

object PreferencePage {

  val PAGE_ID = "de.isse.kiv.ui.preferencePage"

  class KivHomeFieldEditor(preference: String, title: String, parent: Composite) extends FieldEditor(preference, title, parent) {

    private var value: String = _

    private var radioBox: Composite = _

    private var pluginButton: Button = _

    private var kivhomeButton: Button = _

    private var customButton: Button = _

    private var customText: Text = _

    private def updateValue(selectedValue: String) {
      value = selectedValue
      if (value == Settings.KIVHOME_DEFAULT) pluginButton.setSelection(true) else if (value == Settings.KIVHOME_ENVIRON) kivhomeButton.setSelection(true) else {
        customButton.setSelection(true)
        customText.setText(value)
      }
      customText.setEnabled(customButton.getSelection)
    }

    protected def adjustForNumColumns(numColumns: Int) {
      val control = getLabelControl
      if (control != null) {
        control.getLayoutData.asInstanceOf[GridData].horizontalSpan = numColumns
      }
      radioBox.getLayoutData.asInstanceOf[GridData].horizontalSpan = numColumns
    }

    protected def doFillIntoGrid(parent: Composite, numColumns: Int) {
      var data: GridData = null
      val label = getLabelControl(parent)
      data = new GridData()
      data.horizontalSpan = numColumns
      label.setLayoutData(data)
      radioBox = new Composite(parent, SWT.NONE)
      val layout = new GridLayout()
      layout.marginWidth = 0
      layout.marginHeight = 0
      layout.horizontalSpacing = FieldEditor.HORIZONTAL_GAP
      layout.numColumns = numColumns
      radioBox.setLayout(layout)
      data = new GridData()
      data.horizontalSpan = numColumns
      radioBox.setLayoutData(data)
      pluginButton = new Button(radioBox, SWT.RADIO)
      try {
        //Warum ist das drin? Wirft ne Fehlermeldung bezüglich falscher PluginId
        //val libraryPluginPath = ResourceLookup.pluginPath(KIVSystem.LIBRARY_PLUGIN_ID)
        pluginButton.setText("From Plugin")
        pluginButton.addSelectionListener(new SelectionAdapter() {

          override def widgetSelected(event: SelectionEvent) {
            val oldValue = value
            value = Settings.KIVHOME_DEFAULT
            fireValueChanged(FieldEditor.VALUE, oldValue, value)
          }
        })
      } catch {
        case e: Exception => {
          pluginButton.setEnabled(false)
          pluginButton.setText("From Plugin (not installed)")
        }
      }
      kivhomeButton = new Button(radioBox, SWT.RADIO)
      kivhomeButton.setText("$KIVHOME")
      kivhomeButton.addSelectionListener(new SelectionAdapter() {

        override def widgetSelected(event: SelectionEvent) {
          val oldValue = value
          value = Settings.KIVHOME_ENVIRON
          fireValueChanged(FieldEditor.VALUE, oldValue, value)
        }
      })
      customButton = new Button(radioBox, SWT.RADIO)
      customButton.setText("Other:")

      customButton.addSelectionListener(new SelectionAdapter() {

        override def widgetSelected(event: SelectionEvent) {
          customText.setEnabled(customButton.getSelection)
          val oldValue = value
          value = customText.getText
          setPresentsDefaultValue(false)
          fireValueChanged(FieldEditor.VALUE, oldValue, value)
        }
      })
      customText = new Text(parent, SWT.SINGLE | SWT.BORDER)
      data = new GridData()
      data.horizontalSpan = numColumns
      data.horizontalAlignment = GridData.FILL
      data.grabExcessHorizontalSpace = true
      customText.setLayoutData(data)
      customText.addFocusListener(new FocusAdapter() {

        override def focusLost(e: FocusEvent) {
          value = customText.getText
        }
      })

    }

    protected def doLoad() {
      updateValue(getPreferenceStore.getString(getPreferenceName))
    }

    protected def doLoadDefault() {
      updateValue(getPreferenceStore.getDefaultString(getPreferenceName))
    }

    protected def doStore() {
      if (value == null) {
        getPreferenceStore.setToDefault(getPreferenceName)
        return
      }
      getPreferenceStore.setValue(getPreferenceName, value)
    }

    def getNumberOfControls(): Int = 1
  }
}

class KIVFontFieldEditor(preference: String, title: String, parent: Composite) extends FieldEditor(preference, title, parent) {

  private var fontInstallButton: Button = _
  private var composite: Composite = _
  private var label: Label = _
  //private var radioBox: Composite = _
  private def updateValue(selectedValue: String) {}
  protected def adjustForNumColumns(numColumns: Int) {}
  protected def doFillIntoGrid(parent: Composite, numColumns: Int) {

    composite = new Composite(parent, SWT.NONE)

    val layout = new GridLayout()
    layout.marginWidth = 0
    layout.marginHeight = 0
    layout.horizontalSpacing = FieldEditor.HORIZONTAL_GAP
    layout.numColumns = numColumns
    composite.setLayout(layout)

    label = new Label(composite, SWT.NONE)
    label.setText("The default fonts on Windows machines are not the best for \nviewing KIV specifications." +
      "\nThe KIV font can be obtained by clicking the button below.\nSave the font and install it on your machine")

    fontInstallButton = new Button(composite, SWT.PUSH)
    fontInstallButton.setText("  Obtain KIV font  ")

    fontInstallButton.addSelectionListener(new SelectionAdapter() {
      override def widgetSelected(e: SelectionEvent) {
        //if (Platform.getOS() == Platform.OS_WIN32)
        copyFont();

      }
    })

  }

  private def copyFont() {
    val shell = PlatformUI.getWorkbench.getModalDialogShellProvider.getShell
    val fd: FileDialog = new FileDialog(shell, SWT.SAVE)
    fd.setText("Save font")
    // fd.setFilterPath("C:/")
    val filterExt = Array("*.ttf")
    fd.setFileName("KIV.ttf")
    fd.setFilterExtensions(filterExt)
    val filename: String = fd.open()

    // Gidon wird das nicht mögen:P
    // Andere Methode für Copy bekannt ?

    try {
      val classLoader = getClass.getClassLoader
      // used to be classLoader.getResourceAsStream but that one catches the exception.

      val url = classLoader.getResource("font/KIV.ttf");
      if (url == null) return

      val inputStream = url.openStream
      if (inputStream == null) return

      val outputStream = new FileOutputStream(filename)
      val buf = Array.ofDim[Byte](1024)
      var i = 0
      i = inputStream.read(buf)
      while (i != -1) {
        outputStream.write(buf, 0, i)
        i = inputStream.read(buf)
      }

      inputStream.close()
      outputStream.close()
    } catch {
      case err: Exception =>
        Console.error.println("Unable to save KIV font: " + err.getMessage)
    }
  }
  protected def doLoad() {}
  protected def doLoadDefault() {}
  protected def doStore() {}
  def getNumberOfControls(): Int = 1
}

class PreferencePage extends FieldEditorPreferencePage(FieldEditorPreferencePage.GRID) with IWorkbenchPreferencePage {

  override def init(workbench: IWorkbench) {
    setPreferenceStore(KIVPlugin.getDefault.getPreferenceStore)
    setDescription("KIV settings\n")
  }

  protected override def createFieldEditors() {
    val toplevel = new Composite(getFieldEditorParent, 0)
    toplevel.setLayout(new GridLayout())
    toplevel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL))
    val kivGroup = new Group(toplevel, SWT.SHADOW_NONE)
    kivGroup.setText("KIV Paramters")
    kivGroup.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, true, false))
    addField(new KivHomeFieldEditor(Settings.KIVHOME, "Library", kivGroup))
    addField(new StringFieldEditor(Settings.STACKSIZE, "Stack size", kivGroup))
    addField(new BooleanFieldEditor(Settings.INTERNAL_EDITOR, "Use Eclipse Editor (not $KIVEDITOR)", kivGroup))
    addField(new BooleanFieldEditor(Settings.EXPERT, "Run KIV in expert mode (exposes some experimental features)", kivGroup))
    addField(new BooleanFieldEditor(Settings.FORK, "Run KIV in separate process", kivGroup))
    val debugGroup = new Group(toplevel, SWT.SHADOW_NONE)
    debugGroup.setText("Output message settings")
    debugGroup.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, true, false))
    addField(new BooleanFieldEditor(Settings.CONSOLE_OUT, "Informative output", debugGroup))
    addField(new BooleanFieldEditor(Settings.CONSOLE_DEBUG, "Debug output", debugGroup))
    addField(new BooleanFieldEditor(Settings.CONSOLE_ERROR, "Error output", debugGroup))
    val fontGroup = new Group(toplevel, SWT.SHADOW_NONE)
    fontGroup.setText("Install KIV font")
    fontGroup.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, true, false))
    addField(new KIVFontFieldEditor("", "", fontGroup))
  }
}
