package de.isse.kiv.ui

import org.eclipse.ui.IWorkbenchPreferencePage
import org.eclipse.ui.IWorkbench
import org.eclipse.jface.preference.FieldEditorPreferencePage
import de.isse.kiv.KIVPlugin
import org.eclipse.swt.widgets.Composite
import org.eclipse.swt.widgets.Group
import org.eclipse.swt.layout.GridLayout
import org.eclipse.swt.layout.GridData
import org.eclipse.swt.SWT
import org.eclipse.jface.preference.StringFieldEditor
import org.eclipse.jface.preference.ColorFieldEditor
import de.isse.kiv.Settings
import org.eclipse.jface.preference.FontFieldEditor
import org.eclipse.jface.preference.BooleanFieldEditor
import org.eclipse.swt.layout.FillLayout
import org.eclipse.swt.events.SelectionListener
import org.eclipse.swt.events.SelectionEvent
import org.eclipse.swt.custom.StackLayout

class SyntaxColoringPreferencePage extends FieldEditorPreferencePage(FieldEditorPreferencePage.GRID) with IWorkbenchPreferencePage {

  override def init(wb: IWorkbench) {
    setPreferenceStore(KIVPlugin.getDefault.getPreferenceStore)
    setDescription("Syntax Color settings\n")
  }

  protected override def createFieldEditors() {

    val toplevel = new Composite(getFieldEditorParent, 0)
    val gridLayout = new GridLayout()
    gridLayout.numColumns = 2
    toplevel.setLayout(gridLayout)
    toplevel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL))

    val selectionGroup = new Group(toplevel, SWT.SHADOW_NONE)
    selectionGroup.setText("Element:")
    selectionGroup.setLayout(new FillLayout())
    selectionGroup.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, true, false))

    val tokenNameList = new org.eclipse.swt.widgets.List(selectionGroup, SWT.V_SCROLL | SWT.SINGLE | SWT.BORDER)

    Settings.TOKEN_NAME_MAP.foreach(f => tokenNameList.add(f._1))

    val stackingGroup = new Group(toplevel, SWT.NONE)
    val stackLayout = new StackLayout()
    stackingGroup.setLayout(stackLayout)
    
    //Create a Group for each Token and asign an unique id
    val groups = Settings.TOKEN_NAME_MAP.map { tokens =>      
      val settingsGroup = new Group(stackingGroup, SWT.NONE)      
      settingsGroup.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, true, false))
      addField(new ColorFieldEditor(tokens._2 + Settings.SYNTAX_COLOR, Settings.SYNTAX_COLOR_STRING, settingsGroup))
      val x = tokens._2 + Settings.SYNTAX_COLOR
      addField(new BooleanFieldEditor(tokens._2 + Settings.SYNTAX_BOLD, Settings.SYNTAX_BOLD_STRING, settingsGroup))
      addField(new BooleanFieldEditor(tokens._2 + Settings.SYNTAX_ITALIC, Settings.SYNTAX_ITALIC_STRING, settingsGroup))
      addField(new BooleanFieldEditor(tokens._2 + Settings.SYNTAX_STRIKETHROUGH, Settings.SYNTAX_STRIKETHROUGH_STRING, settingsGroup))
      addField(new BooleanFieldEditor(tokens._2 + Settings.SYNTAX_UNDERLINED, Settings.SYNTAX_UNDERLINED_STRING, settingsGroup))
      tokens._1 -> settingsGroup
    }
    tokenNameList.select(0)
    stackLayout.topControl = groups.get(tokenNameList.getItem(tokenNameList.getSelectionIndex)).get

    tokenNameList.addSelectionListener(new SelectionListener() {
      def widgetDefaultSelected(e: SelectionEvent) {

      }
      def widgetSelected(e: SelectionEvent) {
        
        stackLayout.topControl = groups.get(tokenNameList.getItem(tokenNameList.getSelectionIndex)).get
        stackingGroup.layout
      }
    })
    
    val settingsGroup = new Group(toplevel, SWT.SHADOW_NONE)
    settingsGroup.setText("Extended Settings:")
    settingsGroup.setLayout(new FillLayout())
    settingsGroup.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, true, false))
    addField(new BooleanFieldEditor(Settings.HIGHLIGHT_VARIABLES, "Highlight variable occurrences", settingsGroup))
  }
}