package de.isse.kiv.ui.handlers

import org.eclipse.core.commands.AbstractHandler
import org.eclipse.core.commands.ExecutionEvent
import org.eclipse.jface.text.ITextSelection
import org.eclipse.ltk.ui.refactoring.RefactoringWizard
import org.eclipse.ltk.ui.refactoring.RefactoringWizardOpenOperation
import org.eclipse.ui.PlatformUI

import de.isse.kiv.resources.ResourceLookup
import de.isse.kiv.resources.ResourceProperties.toResourceProperties
import de.isse.kiv.refactoring.CodeRenameRefactoring
import de.isse.kiv.source.TokenMap
import de.isse.kiv.ui.wizards.RenameRefactoringWizard
import de.isse.kiv.ui.wizards.ExtractRefactoringWizard
import de.isse.kiv.ui.wizards.MoveRefactoringWizard
import de.isse.kiv.source._
import java.lang.reflect.InvocationTargetException
import de.isse.kiv.ui.editors.FileEditor.openFile
import org.eclipse.ltk.core.refactoring.Refactoring
import de.isse.kiv.refactoring.CodeExtractRefactoring
import de.isse.kiv.refactoring.CodeMoveRefactoring

class CodeRefactoringHandler extends AbstractHandler {

  override def execute(event: ExecutionEvent): AnyRef = {
    val editor = ResourceLookup.activeTextEditor
    if (editor == null) return null
    val cursorOffset = editor.getSelectionProvider().getSelection.asInstanceOf[ITextSelection].getOffset
    val file = ResourceLookup.documentResource
    if (file == null) return null
    val fileModel = file.fileModel
    if (fileModel == null) return null

    var refactorer: Refactoring = null
    val command = event.getCommand.getId.toString
    val provider = editor.getSelectionProvider()
    val selection = provider.getSelection()
    val refactoringWizard = command match {
      case "de.isse.kiv.ui.commands.RefactorRename" =>
        val kivToken = fileModel.tokenAt(cursorOffset).getOrElse(return null)
        if (! kivToken.canRename) return null
        refactorer = new CodeRenameRefactoring(editor, kivToken, file, fileModel)
        new RenameRefactoringWizard(refactorer)

      case "de.isse.kiv.ui.commands.RefactorExtract" =>
        refactorer = new CodeExtractRefactoring(3, editor, selection, file, fileModel)
        new ExtractRefactoringWizard(refactorer)

      case "de.isse.kiv.ui.commands.RefactorMove" =>
        refactorer = new CodeMoveRefactoring(editor, selection, file)
        new MoveRefactoringWizard(refactorer)

      case _ =>
        return null
    }

    val refactoringWizardOpenOperation = new RefactoringWizardOpenOperation(refactoringWizard)

    val shell = PlatformUI.getWorkbench.getModalDialogShellProvider.getShell
    refactoringWizardOpenOperation.run(shell, "")
    if (command == "de.isse.kiv.ui.commands.RefactorMove") {
      val moveRefactorer = refactorer.asInstanceOf[CodeMoveRefactoring]
      openFile(moveRefactorer.selectedSpecFile, moveRefactorer.getRangeFromInsertLocation);
    }

    null
  }
}
