package de.isse.kiv.ui.handlers

import org.eclipse.core.commands.AbstractHandler
import org.eclipse.core.commands.ExecutionEvent
import org.eclipse.jface.text.ITextSelection

import de.isse.kiv.resources.ResourceLookup
import de.isse.kiv.resources.ResourceProperties.toResourceProperties
import de.isse.kiv.ui.Console
import de.isse.kiv.ui.editors.FileEditor.openFile

class OpenSpecHandler extends AbstractHandler {

  override def execute(event: ExecutionEvent): AnyRef = {

    val textEditor = ResourceLookup.activeTextEditor
    if (textEditor == null) return null

    val selectionProvider = textEditor.getSelectionProvider
    if (selectionProvider == null) return null

    val selection = selectionProvider.getSelection.asInstanceOf[ITextSelection]
    if (selection == null) return null

    val file = ResourceLookup.documentResource
    if (file == null) return null

    val filemodel = file.fileModel
    if (filemodel == null) return null

    val kivToken = filemodel.tokenAt(selection.getOffset).getOrElse(return null)

    val activeProjekt = ResourceLookup.activeProject
    if (activeProjekt == null) return null

    val projectModel = activeProjekt.projectModel
    if (projectModel == null) return null

    val specNames = projectModel.specNames
    if (specNames == null) return null    
    

    if (specNames contains kivToken.stringPresentation) {
      val specFile = ResourceLookup.specFile(kivToken.stringPresentation)
      if (specFile.exists()) {
        openFile(specFile)

      } else {
        Console.error.println("Can't open file: " + specFile + "\nNo such file");
      }

    }
    null
  }


}