package de.isse.kiv.ui.navigator

import java.io.IOException
import java.net.URL
import org.eclipse.core.resources.IFile
import org.eclipse.core.resources.IProject
import org.eclipse.jface.resource.ImageDescriptor
import org.eclipse.jface.viewers.ILabelProvider
import org.eclipse.jface.viewers.ILabelProviderListener
import org.eclipse.jface.viewers.LabelProvider
import org.eclipse.swt.graphics.Image
import org.eclipse.ui.ISharedImages
import org.eclipse.ui.PlatformUI
import org.eclipse.ui.ide.IDE.SharedImages
import org.eclipse.ui.navigator.IDescriptionProvider
import de.isse.kiv.KIVPlugin
import de.isse.kiv.ui.Console
import de.isse.kiv.resources.ResourceLookup
import de.isse.kiv.resources.Marker
import org.eclipse.core.resources.IResource

/* org.eclipse.swt.widgets.Tree supports only image + label.
 * Hence we cannot create a custom navigator with say some buttons in a row for one entry.
 * Bullshit.
 * 
 * Alternative: we can override creatCommonViewer in Navigator with stuff from the https://eclipse.org/nebula/ project
 */
object NavigatorLabelProvider {
  val projectImage = PlatformUI.getWorkbench.getSharedImages.getImage(SharedImages.IMG_OBJ_PROJECT)
  val closedProjectImage = PlatformUI.getWorkbench.getSharedImages.getImage(SharedImages.IMG_OBJ_PROJECT_CLOSED)
}

class NavigatorLabelProvider extends LabelProvider with ILabelProvider with IDescriptionProvider {
  override def getText(element: AnyRef): String = {
    element match {
      case project: IProject =>
        project.getName
      case file: IFile =>
        file.getParent.getName + " " + file.getName.dropRight(".utf8".length)
      case _ =>
        null
    }
  }

  def getDescription(element: AnyRef): String = " "

  override def getImage(element: AnyRef): Image = {
    element match {
      case project: IProject =>
        if (project.isOpen())
          NavigatorLabelProvider.projectImage
        else
          NavigatorLabelProvider.closedProjectImage
          
      case file: IFile if file.getName == "specification.utf8" => {
        if (file.findMarkers(Marker.ERROR_MARKER_ID, true, IResource.DEPTH_INFINITE).size == 0)
          ResourceLookup.KIVIcon
        else
          ResourceLookup.KIVError

      }
      case file: IFile if file.getName == "sequents.utf8" => {
        if (file.findMarkers(Marker.ERROR_MARKER_ID, true, IResource.DEPTH_INFINITE).size == 0)
          ResourceLookup.KIVIconGreen
        else
          ResourceLookup.KIVError

      }
    }
  }

  override def addListener(listener: ILabelProviderListener) {
  }

  override def dispose() {
  }

  override def isLabelProperty(element: AnyRef, property: String): Boolean = false

  override def removeListener(listener: ILabelProviderListener) {
  }
}
