package de.isse.kiv.ui.outline

import org.eclipse.jface.viewers.{ITreeSelection, SelectionChangedEvent}
import org.eclipse.ui.views.contentoutline.ContentOutlinePage
import de.isse.kiv.resources.ResourceProperties.toResourceProperties
import de.isse.kiv.ui.editors.FileEditor
import de.isse.kiv.ui.viewers.ViewerProperties._
import org.eclipse.swt.SWT
import org.eclipse.swt.widgets.Composite
import kiv.spec.Theorem
import kiv.prog.Anydeclaration
import kiv.parser.PreOperationDeclaration

object FileOutlinePage {
  val FILE_OUTLINE_ID = "de.isse.kiv.ui.outline.fileEditor"
}

class FileOutlinePage(editor: FileEditor) extends ContentOutlinePage {

  override def createControl(parent: Composite) {
    super.createControl(parent)

    val file = editor.getFile
    if (file == null) return

    val viewer = getTreeViewer

    file.getName match {
      case "specification.utf8" =>
        viewer.setContentProvider(new SpecOutlineContentProvider())
      case "sequents.utf8" =>
        viewer.setContentProvider(new TheoremsOutlineContentProvider())
      case _ =>
        viewer.setContentProvider(NullTreeContentProvider)
    }
    
    viewer.setLabelProvider(new FileOutlineLabelProvider())
    viewer.addSelectionChangedListener(this)

    val site = editor.getSite
    // viewer.createContextMenu(FileOutlinePage.FILE_OUTLINE_ID, site)

    viewer.setInput(file.fileModel)
  }

  private def selectionChanged(selection: ITreeSelection) {
    selection.getFirstElement match {
      case theorem: Theorem =>
        val loc = theorem.location
        editor.setHighlightRange(loc.position, loc.length, true)
      case dec: Anydeclaration =>
        val loc = dec.location
        editor.setHighlightRange(loc.position, loc.length, true)
      case predecl: PreOperationDeclaration =>
        val loc = predecl.nameandloc.loc
        editor.setHighlightRange(loc.position, loc.length, true)
      case _ => editor.resetHighlightRange()
    }
  }

  override def selectionChanged(event: SelectionChangedEvent) {
    super.selectionChanged(event)

    if (event.getSelection.isEmpty) {
      editor.resetHighlightRange()
    } else {
      event.getSelection match {
        case selection: ITreeSelection =>
          selectionChanged(selection)
        case _ =>
      }
    }
  }
}
