package de.isse.kiv.ui.wizards

import org.eclipse.jface.wizard.WizardPage
import org.eclipse.swt.SWT
import org.eclipse.swt.widgets._
import org.eclipse.swt.events._
import org.eclipse.core.resources._
import org.eclipse.ui.PlatformUI
import org.eclipse.swt.layout._
import java.nio.file.Paths
import java.nio.file.Path

class ExportProjectWizardPage(project: IProject) extends WizardPage("Export KIV Project " + project.getName) {
  private var locationField: Combo = _

  setTitle("Export KIV Project " + project.getName)
  setDescription("In order to export the KIV project " + project.getName + " select a file name for the ZIP archive.")

  override def createControl(parent: Composite) {
    val composite = new Composite(parent, SWT.NULL)
    composite.setLayout(new GridLayout(2, false))
    composite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL))
    val locationLabel = new Label(composite, SWT.NONE)
    locationLabel.setText("&Location")
    locationField = new Combo(composite, SWT.BORDER)
    locationField.setText("")
    locationField.addModifyListener(new ModifyListener() {
      override def modifyText(e: ModifyEvent) {
        setPageComplete(validatePage())
      }
    })
    var data = new GridData(GridData.FILL_HORIZONTAL)
    data.widthHint = ImportProjectWizardPage.SIZING_TEXT_FIELD_WIDTH
    data.horizontalSpan = 1
    locationField.setLayoutData(data)
    val browseButton = new Button(composite, SWT.PUSH)
    browseButton.setText("B&rowse...")
    browseButton.addSelectionListener(new SelectionAdapter() {
      override def widgetSelected(event: SelectionEvent) {
        handleLocationBrowseButtonPressed()
      }
    })
    setPageComplete(validatePage())
    setErrorMessage(null)
    setMessage(null)
    setControl(composite)
  }

  def handleLocationBrowseButtonPressed() {
    val shell = PlatformUI.getWorkbench.getModalDialogShellProvider.getShell
    val dialog = new FileDialog(shell, SWT.SAVE)
    dialog.setFileName(project.getName + ".zip")
    dialog.setFilterExtensions(Array("*.zip"))
    dialog.setFilterPath(locationField.getText)
    var selectedPath = dialog.open()
    if (selectedPath == null) selectedPath = ""
    locationField.setText(selectedPath)
  }

  protected def validatePage(): Boolean = {
    val filename = locationField.getText
    if (! filename.endsWith(".zip")) {
      setErrorMessage("The file name does not have the extension .zip")
      return false
    }
    val path = Paths.get(filename)
    val parent = path.getParent
    if (! parent.toFile().exists()) {
      setErrorMessage("Parent directory " + parent + " does not exist")
      return false
    }
    setErrorMessage(null)
    return true
  }

  def getPath: Path = Paths.get(locationField.getText)
}
