package de.isse.kiv.ui.wizards

import org.eclipse.jface.dialogs.Dialog
import org.eclipse.ltk.ui.refactoring.UserInputWizardPage
import org.eclipse.swt.SWT
import org.eclipse.swt.events.ModifyEvent
import org.eclipse.swt.events.ModifyListener
import org.eclipse.swt.events.SelectionEvent
import org.eclipse.swt.events.SelectionListener
import org.eclipse.swt.layout.GridData
import org.eclipse.swt.layout.GridLayout
import org.eclipse.swt.widgets.Button
import org.eclipse.swt.widgets.Composite
import org.eclipse.swt.widgets.Label
import org.eclipse.swt.widgets.Text
import org.eclipse.swt.widgets.Combo
import org.eclipse.swt.widgets.List
import de.isse.kiv.resources.ResourceProperties._

import de.isse.kiv.refactoring.CodeMoveRefactoring

class MoveRefactoringWizardPage(name: String, moveWizard: MoveRefactoringWizard) extends UserInputWizardPage(name: String) {

  var initialized = false
  var combobox: Combo = _

  def createControl(parent: Composite): Unit = {

    initializeDialogUnits(parent)
    val composite = new Composite(parent, SWT.NONE)
    setControl(composite)
    val gridLayout = new GridLayout(1, false)
    gridLayout.verticalSpacing = 8
    composite.setLayout(gridLayout)

  }

  override def setVisible(visible: Boolean) {
    if (visible && !initialized) {
      initialized = true
      val composite = getControl.asInstanceOf[Composite]
      createUserInterface(composite)
      Dialog.applyDialogFont(composite)
    }
    super.setVisible(visible)
  }

  def createUserInterface(composite: Composite) {

    val gridData = new GridData(SWT.FILL, SWT.FILL, true, false)

    gridData.verticalIndent = 8
    composite.setLayoutData(gridData)
    val gridLayout2 = new GridLayout(1, false)
    gridLayout2.verticalSpacing = 8
    gridLayout2.marginTop = 3
    composite.setLayout(gridLayout2)

    val refactorer = moveWizard.getRefactoring.asInstanceOf[CodeMoveRefactoring]

    val label = new Label(composite, SWT.NONE)
    label.setText("Select the specification, where you want to move the procedure to:")
    combobox = new Combo(composite, SWT.DROP_DOWN)
    val files: Array[String] = refactorer.relatedSpecFiles.map { file => file.specName }.toArray
    combobox.setItems(files: _*)

    combobox.addSelectionListener(new SelectionListener() {

      override def widgetSelected(event: SelectionEvent) {
        refactorer.selectedSpecFile = refactorer.relatedSpecFiles.find { file => file.specName == combobox.getItem(combobox.getSelectionIndex()) }.get
      }
      override def widgetDefaultSelected(event: SelectionEvent) {
        refactorer.selectedSpecFile = refactorer.relatedSpecFiles.find { file => file.specName == combobox.getItem(combobox.getSelectionIndex()) }.get
      }
    })
  }
}
